import sys
from os import listdir, path as osp

from cubicweb import Binary



def import_directory(cw, directory, mapfolders=True, bfss=False,
                     fsencoding=sys.stdout.encoding or 'utf-8', quiet=False,
                     _parent=None, _indent='', _alreadyimported=None):
    cw.set_pool()
    assert bfss, 'not implemented'
    if bfss:
        cw.transaction_data['fs_importing'] = True
    if _parent is None:
        directory = osp.abspath(osp.normpath(directory))
    # getting directory content will check it's validity, do it before creating
    # any Folder entity
    dirfiles = listdir(directory)
    # XXX what if not bfss
    if mapfolders:
        _parent = folder_entity(cw, directory, parent=_parent, fsencoding=fsencoding)
        _alreadyimported = set(x.getvalue() for x, in cw.execute(
            "Any fspath(D) WHERE X data D, X filed_under F, F eid %(f)s",
            {'f': _parent.eid}))
    elif _alreadyimported is None:
        _alreadyimported = set(x.getvalue() for x, in cw.execute(
            "Any fspath(D) WHERE X data D, X is File"))
    if not quiet:
        print _indent + '** importing directory', directory
    for fname in dirfiles:
        fpath = osp.join(directory, fname)
        if osp.isdir(fpath):
            import_directory(cw, fname, mapfolders, bfss, fsencoding, quiet,
                             _parent=_parent, _indent=_indent + '  ')
        else:
            # XXX check file already imported when not mapping
            if fpath in _alreadyimported:
                if not quiet:
                    print _indent + '  skipping already imported', fname
                continue
            if not quiet:
                print _indent + '  importing', fname
            if isinstance(fname, str):
                fname = unicode(fname, fsencoding)
            if bfss:
                efile = cw.create_entity('File', data_name=fname,
                                         data=Binary(fpath))
            else:
                stream = open(fpath, 'rb')
                efile = cw.create_entity('File', data_name=fname,
                                         data=Binary(stream.read()))
                stream.close()
            if mapfolders:
                efile.set_relations(filed_under=_parent)


def folder_entity(cw, directory, parent=None, fsencoding='utf-8'):
    if isinstance(directory, str):
        directory = unicode(directory, fsencoding)
    if parent is None:
        rset = cw.execute('Folder X WHERE X name %(name)s', {'name': directory})
    else:
        rset = cw.execute('Folder X WHERE X name %(name)s, X filed_under P, P eid %(p)s',
                          {'name': osp.dirname(directory), 'p': parent.eid})
    if rset:
        return rset.get_entity(0, 0)
    # create the folder
    if parent is None:
        return cw.create_entity('Folder', name=directory)
    return cw.create_entity('Folder', name=directory, filed_under=parent)



# -*- coding: utf-8 -*-

import locale

from os.path import join, dirname
from PIL.Image import open as pilopen

from cubicweb import Binary
from cubicweb.devtools.testlib import CubicWebTC


class FileTC(CubicWebTC):

    def test_set_mime_and_encoding(self):
        fobj = self.request().create_entity('File', data_name=u"foo.txt", data=Binary("xxx"))
        self.assertEqual(fobj.data_format, u'text/plain')
        self.assertEqual(fobj.data_encoding, self.request().encoding)

    def test_set_mime_and_encoding_gz_file(self):
        fobj = self.request().create_entity('File', data_name=u"foo.txt.gz", data=Binary("xxx"))
        self.assertEqual(fobj.data_format, u'text/plain')
        self.assertEqual(fobj.data_encoding, u'gzip')
        fobj = self.request().create_entity('File', data_name=u"foo.txt.gz", data=Binary("xxx"),
                               data_format='application/gzip')
        self.assertEqual(fobj.data_format, u'text/plain')
        self.assertEqual(fobj.data_encoding, u'gzip')
        fobj = self.request().create_entity('File', data_name=u"foo.gz", data=Binary("xxx"))
        self.assertEqual(fobj.data_format, u'application/gzip')
        self.assertEqual(fobj.data_encoding, None)


    def test_set_mime_and_encoding_bz2_file(self):
        fobj = self.request().create_entity('File', data_name=u"foo.txt.bz2", data=Binary("xxx"))
        self.assertEqual(fobj.data_format, u'text/plain')
        self.assertEqual(fobj.data_encoding, u'bzip2')
        fobj = self.request().create_entity('File', data_name=u"foo.txt.bz2", data=Binary("xxx"),
                               data_format='application/bzip2')
        self.assertEqual(fobj.data_format, u'text/plain')
        self.assertEqual(fobj.data_encoding, u'bzip2')
        fobj = self.request().create_entity('File', data_name=u"foo.bz2", data=Binary("xxx"))
        self.assertEqual(fobj.data_format, u'application/bzip2')
        self.assertEqual(fobj.data_encoding, None)

    def test_set_mime_and_encoding_unknwon_ext(self):
        fobj = self.request().create_entity('File', data_name=u"foo.123", data=Binary("xxx"))
        self.assertEqual(fobj.data_format, u'application/octet-stream')
        self.assertEqual(fobj.data_encoding, None)


class ImageTC(CubicWebTC):

    @property
    def data(self):
        return file(join(dirname(__file__), 'data', '20x20.gif')).read()

    def test_resize_image(self):
        # check no resize
        img = self.request().create_entity('File', data=Binary(self.data), data_name=u'20x20.gif')
        self.assertEqual(img.data_format, u'image/gif')
        self.assertEqual(img.data.getvalue(), self.data)
        # check thumb
        self.set_option('image-thumb-size', '5x5')
        pilthumb = pilopen(img.cw_adapt_to('IImage').thumbnail(shadow=False))
        self.assertEqual(pilthumb.size, (5, 5))
        # check resize 10x5
        self.set_option('image-max-size', '10x5')
        img = self.request().create_entity('File', data=Binary(self.data), data_name=u'20x20.gif')
        self.assertEqual(img.data_format, u'image/gif')
        pilimg = pilopen(img.data)
        self.assertEqual(pilimg.size, (5, 5))
        # also on update
        img.set_attributes(data=Binary(self.data))
        img.clear_all_caches()
        pilimg = pilopen(img.data)
        self.assertEqual(pilimg.size, (5, 5))
        # test image smaller than max size
        self.set_option('image-max-size', '40x40')
        img.set_attributes(data=Binary(self.data))
        pilimg = pilopen(img.data)
        self.assertEqual(pilimg.size, (20, 20))

if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



from os.path import dirname, join, abspath
from cubicweb.devtools.testlib import CubicWebTC

from cubicweb.server.sources import storages

from cubes.file.fsimport import import_directory

HERE = dirname(__file__)

STORAGE = storages.BytesFileSystemStorage('whatever')

class importDirectoryTC(CubicWebTC):

    def setup_database(self):
        storages.set_attribute_storage(self.repo, 'File', 'data', STORAGE)

    def tearDown(self):
        storages.unset_attribute_storage(self.repo, 'File', 'data')

    def test_folder_bfss(self):
        import_directory(self.session, join(HERE, 'data', 'toimport'),
                         bfss=True, quiet=True)
        self.assertEqual(self.execute('Any COUNT(F) WHERE F is Folder')[0][0], 1)
        self.assertEqual(self.execute('Any COUNT(F) WHERE F is File')[0][0], 2)
        fpath = self.execute('Any fspath(D) WHERE F is File, F data D, F data_name "coucou.txt"')[0][0]
        self.assertEqual(fpath.getvalue(), join(abspath(HERE), 'data', 'toimport', 'coucou.txt'))
        f = self.execute('Any F WHERE F is File, F data_name "coucou.txt"').get_entity(0, 0)
        self.assertEqual(f.data.getvalue(), 'bijour\n')
        self.assertEqual(len(f.filed_under), 1)
        self.assertEqual(f.filed_under[0].name, join(abspath(HERE), 'data', 'toimport'))
        # test reimport
        import_directory(self.session, join(HERE, 'data', 'toimport'),
                         bfss=True, quiet=True)
        self.assertEqual(self.execute('Any COUNT(F) WHERE F is Folder')[0][0], 1)
        self.assertEqual(self.execute('Any COUNT(F) WHERE F is File')[0][0], 2)
        # test import of an unexistant directory
        self.assertRaises(OSError, import_directory, self.session, join(HERE, 'pouet'), bfss=True, quiet=True)
        self.assertEqual(self.execute('Any COUNT(F) WHERE F is Folder')[0][0], 1)

if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()




from cubicweb.devtools.testlib import CubicWebTC

from cubicweb import ValidationError, NoSelectableObject, Binary
from cubicweb.interfaces import IDownloadable


class FileTC(CubicWebTC):

    def setup_database(self):
        self.fobj = self.request().create_entity('File', data_name=u"foo.pdf",
                                                 data=Binary("xxx"),
                                                 data_format=self.mime_type)

    icon = 'text.ico'
    mime_type = u"text/plain"

    def test_idownloadable(self):
        idownloadable = self.fobj.cw_adapt_to('IDownloadable')
        self.assertEqual(idownloadable.download_data(), 'xxx')
        self.assertEqual(idownloadable.download_url(),
                          u'http://testing.fr/cubicweb/%s/%s?vid=download' % (
            self.fobj.__regid__.lower(), self.fobj.eid))
        self.assertEqual(idownloadable.download_content_type(), self.mime_type)

    def test_base(self):
        self.assertEqual(self.fobj.size(), 3)
        self.assertEqual(self.fobj.icon_url(),
                          'http://testing.fr/cubicweb/data/icons/'+self.icon)

    def test_views(self):
        self.vreg['views'].select('download', self.fobj._cw, rset=self.fobj.cw_rset)
        self.fobj.view('gallery')
        self.assertRaises(NoSelectableObject, self.fobj.view, 'image')
        self.assertRaises(NoSelectableObject, self.fobj.view, 'album')


class ImageTC(FileTC):
    icon = 'image_png.ico'
    mime_type = u"image/png"

    def setup_database(self):
        self.fobj = self.request().create_entity('File', data_name=u"foo.png", data=Binary("xxx"),
                                    data_format=self.mime_type)

    def test_views(self):
        self.vreg['views'].select('download', self.fobj._cw, rset=self.fobj.cw_rset)
        self.fobj.view('gallery')
        self.fobj.view('image')
        self.fobj.view('album')
        # XXX test thumbnail generation


class MimeTypeDetectionTC(CubicWebTC):

    def test_extra_dot(self):
        fobj = self.request().create_entity('File', data_name=u"foo.toto.pdf",
                                                 data=Binary("xxx"))
        self.assertEqual(fobj.data_format, 'application/pdf')


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()
